/*  
 * KLDP Zilla Logic Part
 * Copyright 2004 (c) Sung Kim <hunkim@cs.ucsc.edu>
 *
 * This file is part of KLDPZilla
 *
 * KLDPZilla is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * KLDPZilla is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GForge; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

const KLDP_RDF_WIKI = "http://wiki.kldp.org/wiki.php/RecentChanges?action=rss_rc";
const KLDP_RDF_KLDPNET = "http://kldp.net/export/rss_sfnews.php";
const KLDP_RDF_BBS = "http://bbs.kldp.org/rss.php";
const KLDP_RDF_GEEK = "http://geekforum.kldp.org/rss.php";

const NUM_OF_RDFS = 4;
/*----------------------- RDF Define ------------------------------*/
function Channel(limit) {
    this.limit = limit;	
    this.title = null;
    this.link = null;
    this.description = null;
    this.items = new Array();
}

Channel.prototype = {
  getTitle: function() {
      return this.title;
  },
  getLink: function() {
      return this.link;
  },
  getDescription: function() {
      return this.description;
  },
  getItems: function() {
      return this.items;
  },
    /* Expecting channel node */
  parseXML: function(node) {
      for (var i=0; i<node.childNodes.length; i++) {
	  sub = node.childNodes[i];
	  
	  /* Expecting firstChild */
	  if (sub.firstChild == null)
	    continue;
	  
	  if (sub.localName == 'title')
	    this.title = sub.firstChild.nodeValue;
	  else if (sub.localName == 'link') 
	    this.link = sub.firstChild.nodeValue;    
	  else if (sub.localName == 'description')
	    this.description = sub.firstChild.nodeValue;    
	  else if (sub.localName == 'item' && this.limit > this.items.length) {
	      var item = new Item();
	      item.parseXML(sub);
	      this.items[this.items.length] = item;
	  }
      }
  } 
}

function Item() {
    this.title = null;
    this.link = null;
}  

Item.prototype = {
  getTitle: function() {
      return this.title;
  }, 
  getLink: function() {
      return this.link;
  },
    /* Expectiong item node */
  parseXML: function(node) {
      for (var i=0; i<node.childNodes.length; i++) {
	  sub = node.childNodes[i];
	  
	  if (sub.firstChild == null)
	    continue;
	  
	  if (sub.localName == 'title')
	    this.title = sub.firstChild.nodeValue;
	  else if (sub.localName == 'link') 
	    this.link = sub.firstChild.nodeValue;    
      }
  }
}

function RDF(limit) {
    this.limit = limit;	
    this.channel = null;
    this.items = new Array();
}

RDF.prototype = {
  getChannel: function() {
      return this.channel;
  }, 
  getItems: function() {
      /* It no itemes, it might be in channel */
      if (this.items.length==0) 
	return this.channel.getItems();
      
      return this.items;
  },
    /* Expecting RDF root */
  parseXML: function(root) {
      for (var i=0; i<root.childNodes.length; i++) {
	  sub = root.childNodes[i];
	  if (sub.localName == 'channel') {
	      this.channel = new Channel(this.limit);
	      this.channel.parseXML(sub);
	  } else if (sub.localName == 'item' && this.limit > this.items.length) {
	      var item = new Item();
	      item.parseXML(sub);
	      this.items[this.items.length] = item;
	  }
      }
  }
}

/*-----------------------------------------------------------------------
HTTP Connection function 
-----------------------------------------------------------------------*/
function parseRDF(root, index) {
    var rdf = new RDF(rdfLimit[index]);
    rdf.parseXML(root);
    return rdf;
}

function getRDFCallback (e) {
    var p = e.currentTarget;
    
    if (p.status != 200) {
	alert("Error:" + p.status);		
    } else {           
    	try {
            setStatus((gRDFArr.length*30 + 30) + '%');
 	    var roottag = p.responseXML.documentElement;
	    gRDFArr[gRDFArr.length] = parseRDF(roottag, gRDFArr.length);
            setStatus((gRDFArr.length*30 + 20) + '%');
            /* We are done here */
            if (gRDFArr.length== NUM_OF_RDFS) {
		 refreshContent(gRDFArr);
                 setStatus('100%');
            } 
	} catch (e) {
	    errorHandle(e);
	}
    }
}

function getRDF(url) {
    p = new XMLHttpRequest();
    /* false for sync */
    p.open("GET", url, true);
    p.onload = getRDFCallback;
    p.send(null);
}

function getIcon(url) {
    if (url.indexOf('http://kldp.net')!=-1) {
	return 'chrome://kldpzilla/content/icons/1.png';
    } else if (url.indexOf('http://wiki.kldp')!=-1) {
	return 'chrome://kldpzilla/content/icons/0.png';
    } else if (url.indexOf('http://bbs.kldp')!=-1) {
	return 'chrome://kldpzilla/content/icons/2.png';
    } else {
        return 'chrome://kldpzilla/content/icons/3.png';
    }
}

/*------------------------------------------------------------------------
Content Update function 
*------------------------------------------------------------------------*/

function rdf_doc_write(rdf, index) {
    
    var channel = rdf.getChannel();
	
	if (!channel)
		return "";
		
    rdf_doc = 'doc.writeln("<tr><td height=\\"5\\"></td></tr>");\n';
    rdf_doc += 'doc.writeln("<tr><td valign=\\"middle\\" align=\\"center\\">");\n';
    rdf_doc += 'doc.writeln("<a href=\\"'+ channel.getLink() + '\\" target=\\"_content\\">");\n';
    rdf_doc += 'doc.writeln("<img border=\\"0\\" height=\\"24\\" src=\\"' + 
	getIcon(channel.getLink()) +
	'\\">");\n';
    rdf_doc += 'doc.writeln("</a></td></tr>");\n';
    
   var items = rdf.getItems();
    for (var i = 0; i < items.length; i++) {
	var item = items[i];
	rdf_doc += 'doc.writeln("<tr><td valign=\\"middle\\">");\n';

	/* URL */
	rdf_doc += 'doc.writeln("<img height=\\"11\\" src=\\"chrome://kldpzilla/content/icons/p' + index + '.gif\\">");\n';
	rdf_doc += 'doc.writeln("<a style=\\"font-size: 11px; color: #663300\\"  href=\\"'+ 
	  item.getLink() + '\\" target=\\"_content\\">");\n';
	rdf_doc += safe_doc_write(item.getTitle());
	rdf_doc += 'doc.writeln("</a></td>");\n';
	
	rdf_doc += 'doc.writeln("</tr>");\n';
    }
    rdf_doc += 'doc.writeln("<tr><td height=\\"2\\" bgcolor=\\"#BBBBBB\\"></td></tr>");\n';
    return rdf_doc;
}

function refreshContent(RDFArr) {
    var doc = document.getElementById("contentiframe").contentDocument;
    var parseCode = 'doc.open();\n';
  
    parseCode+= 'doc.writeln("<html><head>");\n';
    parseCode+=	'doc.writeln("</head>");\n';
    parseCode+= 'doc.writeln("<body bgcolor=\\"#FFFFF\\">");\n';
	parseCode+= 'doc.writeln("<table CELLPADDING=\\"0\\" CELLSPACING=\\"0\\" border=\\"0\\" width=\\"100%\\">");\n';
    
	parseCode += 'doc.writeln("<tr><td height=\\"5\\"></td></tr>");\n';
    parseCode += 'doc.writeln("<tr><td valign=\\"middle\\" align=\\"center\\">");\n';
    parseCode += 'doc.writeln("<a href=\\"http://kldp.org\\" target=\\"_content\\">");\n';
    parseCode += 'doc.writeln("<img border=\\"0\\" src=\\"chrome://kldpzilla/content/icons/kldp.jpg\\">");\n';
    parseCode += 'doc.writeln("</a></td></tr>");\n';
    parseCode += 'doc.writeln("<tr><td height=\\"2\\" bgcolor=\\"#BBBBBB\\"></td></tr>");\n';	
    
	for (var i = 0; i < RDFArr.length; i++) {
		var rdf = RDFArr[i];
		parseCode += rdf_doc_write(rdf, i);
    }
    
    parseCode += 'doc.writeln("</table>");\n';
    parseCode += 'doc.writeln("<form name=\\"buf\\">");\n';
    parseCode += 'doc.writeln("<input name=\\"action\\" type=\\"hidden\\">");\n';
    parseCode += 'doc.writeln("<input name=\\"id\\" type=\\"hidden\\">");\n';
    parseCode += 'doc.writeln("</form></body></html>");\n';
    parseCode += 'doc.close();\n';
    
    eval(parseCode);
}

/* Main Function */

var gRDFArr = new Array();
var rdfLimit = new Array(5, 5, 100, 5);
function refresh() {
    /* Get RDFs*/
    try {
    gRDFArr = new Array();
  
    getRDF(KLDP_RDF_WIKI);
    getRDF(KLDP_RDF_KLDPNET);
    getRDF(KLDP_RDF_BBS);      
    getRDF(KLDP_RDF_GEEK);

    setStatus('10%');	
    } catch (e) {
	errorHandle(e);
    }
}	
 
/*------------------------------------------------------------------
 * Utility functions 
 *------------------------------------------------------------------*/
/* general function for status access */
function setStatus(percent)
{
   // alert(percent);	
    var meter = document.getElementById("progress");
    if (percent=='0%') { 	
       meter.setAttribute('collapsed', false);
       meter.setAttribute('mode', 'undetermined');
       document.getElementById("refresh").disabled = true;	
    } if (percent=='100%') {
       meter.setAttribute('mode', 'determined');
       meter.setAttribute('value', '100%');
       meter.setAttribute('collapsed', true);
       document.getElementById("refresh").disabled = false;	
    } else {
        meter.setAttribute('value', percent);
    }
}

/*
** Error handle utility
*/
function errorHandle(err) {
    alert(err.name + ":" + err + "\n"+err.fileName + ":" + err.lineNumber);
}

/* remove CRLF and quotation marks */
function safe_doc_write(str) {
    var ret = '';
    var lineseparator = /\n/; 
    var lines = str.split(lineseparator);
    
    for (var i=0; i<lines.length; i++) {
	var safe_line = lines[i].replace(/"/gi, "'"); 
	ret+='doc.writeln("'+safe_line+'");\n';
    }
    return ret;
}














